@@ -16,11 +16,7 @@ except ImportError:
 
     class immutables:  # type: ignore
         class Map(dict):
-            def update(self, *args, **kwargs):
-                raise TypeError
-
-            def __setitem__(self, key, value):
-                raise TypeError
+            pass
 
 
 from .pickablenv import PickableRecordset
