@@ -6,21 +6,20 @@
 #
 # This is free software; you can do what the LICENCE file allows you to.
 #
+from __future__ import annotations
+
+import typing as t
 from dataclasses import dataclass
 from enum import Enum
-from typing import Generic, List, Optional, Tuple, Type, TypeVar
 
-from xotl.tools.infinity import Infinity
+from xotl.tools.infinity import Infinity, InfinityType
 from xotl.tools.symbols import Undefined
 
-from .types import EqTypeClass, OrdTypeClass
-
-TEq = TypeVar("TEq", bound=EqTypeClass)
-TOrd = TypeVar("TOrd", bound=OrdTypeClass)
+from .types import TOrd
 
 
 @dataclass(init=False)
-class Range(Generic[TOrd]):
+class Range(t.Generic[TOrd]):
     """A range containing values between `lowerbound` and `upperbound`.
 
     Boundaries can be instances of `Included`:class: or `Excluded`:class:
@@ -50,8 +49,8 @@ class Range(Generic[TOrd]):
 
     """
 
-    _lower: "Bound[TOrd]"
-    _upper: "Bound[TOrd]"
+    _lower: Bound[TOrd]
+    _upper: Bound[TOrd]
 
     __slots__ = ("_lower", "_upper")
 
@@ -64,9 +63,7 @@ class Range(Generic[TOrd]):
         return self._upper.bound
 
     @classmethod
-    def new_closed(
-        cls, lowerbound: Optional[TOrd], upperbound: Optional[TOrd]
-    ) -> "Range[TOrd]":
+    def new_closed(cls, lowerbound: t.Optional[TOrd], upperbound: t.Optional[TOrd]) -> Range[TOrd]:
         """Return a closed range if both boundaries allow it.
 
         If any of the boundaries is None, or an Infinity value, the result
@@ -78,9 +75,7 @@ class Range(Generic[TOrd]):
         return Range(lb, ub)
 
     @classmethod
-    def new_open(
-        cls, lowerbound: Optional[TOrd], upperbound: Optional[TOrd]
-    ) -> "Range[TOrd]":
+    def new_open(cls, lowerbound: t.Optional[TOrd], upperbound: t.Optional[TOrd]) -> Range[TOrd]:
         """Return an open range."""
         lb = Bound.normalized_lower_bound(lowerbound, Excluded)
         ub = Bound.normalized_upper_bound(upperbound, Excluded)
@@ -88,8 +83,10 @@ class Range(Generic[TOrd]):
 
     @classmethod
     def new_open_left(
-        cls, lowerbound: Optional[TOrd], upperbound: Optional[TOrd]
-    ) -> "Range[TOrd]":
+        cls,
+        lowerbound: t.Optional[TOrd],
+        upperbound: t.Optional[TOrd],
+    ) -> Range[TOrd]:
         """Return a range open on the lower boundary and possibly closed in the upper boundary."""
         lb = Bound.normalized_lower_bound(lowerbound, Excluded)
         ub = Bound.normalized_upper_bound(upperbound, Included)
@@ -97,22 +94,18 @@ class Range(Generic[TOrd]):
 
     @classmethod
     def new_open_right(
-        cls, lowerbound: Optional[TOrd], upperbound: Optional[TOrd]
-    ) -> "Range[TOrd]":
+        cls,
+        lowerbound: t.Optional[TOrd],
+        upperbound: t.Optional[TOrd],
+    ) -> Range[TOrd]:
         """Return a range possibly closed on the lower boundary and open in the upper boundary."""
         lb = Bound.normalized_lower_bound(lowerbound, Included)
         ub = Bound.normalized_upper_bound(upperbound, Excluded)
         return Range(lb, ub)
 
-    def __init__(
-        self, lowerbound: "Bound[TOrd]", upperbound: "Bound[TOrd]"
-    ) -> None:
-        lowerbound = (
-            lowerbound if lowerbound is not None else Excluded(-Infinity)
-        )
-        upperbound = (
-            upperbound if upperbound is not None else Excluded(Infinity)
-        )
+    def __init__(self, lowerbound: Bound[TOrd], upperbound: Bound[TOrd]) -> None:
+        lowerbound = lowerbound if lowerbound is not None else Excluded(t.cast(TOrd, -Infinity))
+        upperbound = upperbound if upperbound is not None else Excluded(t.cast(TOrd, Infinity))
         if not isinstance(lowerbound, Bound):
             raise ValueError(
                 f"lowerbound ({lowerbound}) is not of type Excluded(), Included() or Unbounded()"
@@ -154,8 +147,7 @@ class Range(Generic[TOrd]):
     def empty(self):
         """True if and only if empty"""
         return self.lowerbound == self.upperbound and (
-            isinstance(self._lower, Excluded)
-            or isinstance(self._upper, Excluded)
+            isinstance(self._lower, Excluded) or isinstance(self._upper, Excluded)
         )
 
     def __contains__(self, value):
@@ -175,7 +167,7 @@ class Range(Generic[TOrd]):
         """True if neither boundary is unbounded."""
         return bool(self._lower) and bool(self._upper)
 
-    def diff(self, other: "Range") -> Tuple["Range", ...]:
+    def diff(self, other: "Range") -> t.Tuple["Range", ...]:
         """The difference of self from other.
 
         The result is two ranges that jointly have all the values in self,
@@ -189,7 +181,7 @@ class Range(Generic[TOrd]):
             return ()
         elif other:
             assert self > other
-            res: Tuple[Range[TOrd], ...] = ()
+            res: t.Tuple[Range[TOrd], ...] = ()
             before = Range(self._lower, ~other._lower)
             if before:
                 res += (before,)
@@ -210,29 +202,22 @@ class Range(Generic[TOrd]):
         """
         assert self & other or (
             self.upperbound == other.lowerbound
-            and (
-                isinstance(self._upper, Included)
-                or isinstance(other._lower, Included)
-            )
+            and (isinstance(self._upper, Included) or isinstance(other._lower, Included))
         ), f"Cannot merge {self} and {other}"
 
         # The join of a `Excluded(a)` bound with `Included(a)` bound
         # must allways include a despite the order.
         # So we need find the max and min `bound` and `type`.
-        self._lower = min(
-            self._lower, other._lower, key=_max_key_by_bound_and_type
-        )
-        self._upper = max(
-            self._upper, other._upper, key=_min_key_by_bound_and_type
-        )
+        self._lower = min(self._lower, other._lower, key=_max_key_by_bound_and_type)
+        self._upper = max(self._upper, other._upper, key=_min_key_by_bound_and_type)
 
-    def join(*ranges_: "Range[TOrd]") -> Tuple["Range[TOrd]", ...]:
+    def join(*ranges_: Range[TOrd]) -> t.Tuple[Range[TOrd], ...]:
         """Return several disjoint ranges that describe the union of `ranges`."""
         # The join of `Range(Excluded(a), b)` with `Range(Included(a),
         # c)` must include a despite the order.  So we need order by
         # lowerbound and type.
         ranges = sorted((r for r in ranges_ if r), key=_by_lowerbound_and_type)
-        result: List[Range] = []
+        result: t.List[Range[TOrd]] = []
         for next_ in ranges:
             if not result:
                 # Don't use the same object since we can modify it with
@@ -254,8 +239,7 @@ class Range(Generic[TOrd]):
                 #                   Range(Excluded(b), Bound(c))
                 #
                 if current & next_ or (
-                    current.upperbound == next_.lowerbound
-                    and current._upper != next_._lower
+                    current.upperbound == next_.lowerbound and current._upper != next_._lower
                 ):
                     current._merge_with(next_)
                 else:
@@ -265,7 +249,7 @@ class Range(Generic[TOrd]):
         else:
             return ()
 
-    def __and__(self, other: "Range") -> "Range":
+    def __and__(self: Range[TOrd], other: Range[TOrd]) -> Range[TOrd]:
         """Get the intersection of self and other."""
         if not isinstance(other, Range):
             return NotImplemented
@@ -307,28 +291,28 @@ class Range(Generic[TOrd]):
         else:
             return hash((Undefined, Undefined))
 
-    def __le__(self, other: "Range") -> bool:  # pragma: no cover
+    def __le__(self, other) -> bool:  # pragma: no cover
         "True if self is a sub-range of other"
         try:
             return self == (self & other)
         except TypeError:
             return NotImplemented
 
-    def __lt__(self, other: "Range") -> bool:  # pragma: no cover
+    def __lt__(self, other) -> bool:  # pragma: no cover
         "True is self is a proper sub-range of other"
         try:
             return self != other and self <= other
         except TypeError:
             return NotImplemented
 
-    def __ge__(self, other: "Range") -> bool:  # pragma: no cover
+    def __ge__(self, other) -> bool:  # pragma: no cover
         "True is self is a super-range of other"
         try:
             return (self & other) == other
         except TypeError:
             return NotImplemented
 
-    def __gt__(self, other: "Range") -> bool:  # pragma: no cover
+    def __gt__(self, other) -> bool:  # pragma: no cover
         "True is self is a proper super-range of other"
         try:
             return self != other and self >= other
@@ -336,7 +320,7 @@ class Range(Generic[TOrd]):
             return NotImplemented
 
     @property
-    def sample(self) -> Optional[TOrd]:
+    def sample(self) -> t.Optional[TOrd]:
         """Return a member of the range.  Return None if the range is empty or OPEN."""
         if self.empty or self.kind == RangeKind.OPEN:
             # There's no general way to get a sample from an OPEN range.
@@ -348,24 +332,16 @@ class Range(Generic[TOrd]):
             return self.upperbound
 
     @property
-    def kind(self) -> "RangeKind":
+    def kind(self) -> RangeKind:
         "The kind of the range."
-        if isinstance(self._lower, Included) and isinstance(
-            self._upper, Excluded
-        ):
+        if isinstance(self._lower, Included) and isinstance(self._upper, Excluded):
             return RangeKind.OPEN_RIGHT
-        elif isinstance(self._lower, Included) and isinstance(
-            self._upper, Included
-        ):
+        elif isinstance(self._lower, Included) and isinstance(self._upper, Included):
             return RangeKind.CLOSED
-        elif isinstance(self._lower, Excluded) and isinstance(
-            self._upper, Excluded
-        ):
+        elif isinstance(self._lower, Excluded) and isinstance(self._upper, Excluded):
             return RangeKind.OPEN
         else:
-            assert isinstance(self._lower, Excluded) and isinstance(
-                self._upper, Included
-            )
+            assert isinstance(self._lower, Excluded) and isinstance(self._upper, Included)
             return RangeKind.OPEN_LEFT
 
     def __str__(self):
@@ -373,7 +349,7 @@ class Range(Generic[TOrd]):
 
 
 @dataclass(unsafe_hash=True, order=True)
-class Bound(Generic[TOrd]):
+class Bound(t.Generic[TOrd]):
     """A boundary condition to be used in Range.
 
     You must use instances of `Included`:class: or `Excluded`:class:.  Calling
@@ -387,8 +363,10 @@ class Bound(Generic[TOrd]):
 
     @classmethod
     def normalized_upper_bound(
-        cls, bound: Optional[TOrd], Boundary: Type["Bound[TOrd]"]
-    ) -> "Bound[TOrd]":
+        cls,
+        bound: t.Optional[TOrd],
+        Boundary: t.Type[Bound[TOrd]],
+    ) -> Bound[TOrd]:
         """Return a normalized instance of Boundary for the upper bound of Range.
 
         If `bound` is either None, False, Infinity or -Infinity, return
@@ -397,15 +375,17 @@ class Bound(Generic[TOrd]):
         """
         assert Boundary is Included or Boundary is Excluded
         assert not isinstance(bound, Bound)
-        if bound is None or bound is False or isinstance(bound, type(Infinity)):
-            return Excluded(Infinity)
+        if bound is None or bound is False or isinstance(bound, InfinityType):
+            return Excluded(t.cast(TOrd, Infinity))
         else:
             return Boundary(bound)
 
     @classmethod
     def normalized_lower_bound(
-        cls, bound: Optional[TOrd], Boundary: Type["Bound[TOrd]"]
-    ) -> "Bound[TOrd]":
+        cls,
+        bound: t.Optional[TOrd],
+        Boundary: t.Type[Bound[TOrd]],
+    ) -> Bound[TOrd]:
         """Return a normalized instance of Boundary for the upper bound of Range.
 
         If `bound` is either None, False, Infinity or -Infinity, return
@@ -414,16 +394,16 @@ class Bound(Generic[TOrd]):
         """
         assert Boundary is Included or Boundary is Excluded
         assert not isinstance(bound, Bound)
-        if bound is None or bound is False or isinstance(bound, type(Infinity)):
-            return Excluded(-Infinity)
+        if bound is None or bound is False or isinstance(bound, InfinityType):
+            return Excluded(t.cast(TOrd, -Infinity))
         else:
             return Boundary(bound)
 
-    def __invert__(self) -> "Bound[TOrd]":
-        ...
+    def __invert__(self) -> Bound[TOrd]:
+        raise NotImplementedError
 
 
-class Included(Bound):
+class Included(Bound[TOrd]):
     "An included boundary in a `Range`:class:."
 
     def __init__(self, bound: TOrd) -> None:
@@ -432,7 +412,7 @@ class Included(Bound):
         super().__init__(bound)
 
     @classmethod
-    def safer_new(cls, bound: TOrd) -> "Bound[TOrd]":
+    def safer_new(cls, bound: TOrd) -> Bound[TOrd]:
         """Try to safely create an Included boundary, falling back to
         `Excluded`:class: for infinite marks.
 
@@ -455,7 +435,7 @@ class Included(Bound):
         return Excluded(self.bound)
 
 
-class Excluded(Bound):
+class Excluded(Bound[TOrd]):
     "An excluded boundary in a `Range`:class:"
 
     def __bool__(self):
@@ -486,7 +466,7 @@ class RangeKind(Enum):
     OPEN = "{lb < a < ub}"
 
 
-def _max_key_by_bound_and_type(boundary: Bound[TOrd]) -> Tuple[TOrd, int]:
+def _max_key_by_bound_and_type(boundary: Bound[TOrd]) -> t.Tuple[TOrd, int]:
     """Return a tuple of `(bound, -1 or 1)`.
 
     The second component is -1 if the boundary is Included, and 1 if Excluded.
@@ -500,7 +480,7 @@ def _max_key_by_bound_and_type(boundary: Bound[TOrd]) -> Tuple[TOrd, int]:
     return (boundary.bound, -1 if isinstance(boundary, Included) else 1)
 
 
-def _min_key_by_bound_and_type(boundary: Bound[TOrd]) -> Tuple[TOrd, int]:
+def _min_key_by_bound_and_type(boundary: Bound[TOrd]) -> t.Tuple[TOrd, int]:
     """Return a tuple of `(bound, -1 or 1)`.
 
     This is the same as _max_key_by_bound_and_type but negating the second
@@ -514,5 +494,5 @@ def _min_key_by_bound_and_type(boundary: Bound[TOrd]) -> Tuple[TOrd, int]:
     return bound, -order
 
 
-def _by_lowerbound_and_type(r: Range[TOrd]) -> Tuple[TOrd, int]:
+def _by_lowerbound_and_type(r: Range[TOrd]) -> t.Tuple[TOrd, int]:
     return _max_key_by_bound_and_type(r._lower)
