@@ -6,49 +6,44 @@
 #
 # This is free software; you can do what the LICENCE file allows you to.
 #
-import unittest
 import operator
 import pickle
-
+import unittest
 from copy import copy
+from datetime import timedelta
 from itertools import combinations
 from random import sample, shuffle
-from datetime import timedelta
 
+from hypothesis import example, given, settings
+from hypothesis import strategies as st
+from hypothesis.strategies import SearchStrategy
 from xotl.tools.infinity import Infinity
 
-from hypothesis import strategies as st, given, example
-from hypothesis.strategies import SearchStrategy
-
 from ..domains import EquivalenceSet, IntervalSet
-from ..ranges import Range, Bound, Included, Excluded, RangeKind
-
+from ..ranges import Bound, Excluded, Included, Range, RangeKind
 from .strategies.domains import (
-    float_ranges,
     date_ranges,
     equivalence_sets,
-    non_empty_equivalence_sets,
-    many_float_ranges,
+    float_ranges,
+    get_date_member_strategy,
     many_date_ranges,
+    many_float_ranges,
     members_of_date_ranges,
     members_of_float_ranges,
+    non_empty_equivalence_sets,
     type_of_bounds,
 )
 
 DAY = timedelta(1)
 
 
-kinds_with_samples = st.sampled_from(
-    (Range.new_open_left, Range.new_open_right, Range.new_closed)
-)
+kinds_with_samples = st.sampled_from((Range.new_open_left, Range.new_open_right, Range.new_closed))
 
 
 @st.composite
 def _sorted_points(draw, many=2):
     assert many >= 2
-    points = list(
-        sorted(draw(st.sets(st.integers(), min_size=many, max_size=many)))
-    )
+    points = list(sorted(draw(st.sets(st.integers(), min_size=many, max_size=many))))
     if draw(st.booleans()):
         points[0] = -Infinity
     if draw(st.booleans()):
@@ -88,9 +83,7 @@ def test_range_intersection_with_a_common_boundary(points, Bound1, Bound2):
     type_of_bounds,
     type_of_bounds,
 )
-def test_range_intersection_with_a_common_Excluded_boundary(
-    points, Bound1, Bound2
-):
+def test_range_intersection_with_a_common_Excluded_boundary(points, Bound1, Bound2):
     a, b, c = sorted(points)
     assert not Range(Bound1(a), Excluded(b)) & Range(Included(b), Bound2(c))
     assert not Range(Bound1(a), Included(b)) & Range(Excluded(b), Bound2(c))
@@ -193,9 +186,9 @@ def test_range_diff_overlapping_ranges(points, Bounds):
     a, b, c, d = zip_map(Bounds, sorted(points))
     r1 = Range(a, c)
     r2 = Range(b, d)
-    before, = r1.diff(r2)
+    (before,) = r1.diff(r2)
     assert before == Range(a, ~b)
-    after, = r2.diff(r1)
+    (after,) = r2.diff(r1)
     assert after == Range(~c, d)
 
 
@@ -266,9 +259,7 @@ def test_float_ranges_generation(data, r0):
 
 @given(st.data())
 def test_float_ranges_generation_regression_job_117109(data):
-    _test_float_ranges_generation(
-        data, Range.new_open(0, 1), kinds=st.just(Range.new_closed)
-    )
+    _test_float_ranges_generation(data, Range.new_open(0, 1), kinds=st.just(Range.new_closed))
 
 
 @given(st.data(), date_ranges())
@@ -317,12 +308,8 @@ class TestRange(DomainCase):
     @staticmethod
     def add(left, right):  # pragma: no cover
         if left and right:
-            assert (
-                right.lowerbound in left or right.lowerbound == left.upperbound
-            )
-            return Range.new_open_right(
-                left.lowerbound, max(left.upperbound, right.upperbound)
-            )
+            assert right.lowerbound in left or right.lowerbound == left.upperbound
+            return Range.new_open_right(left.lowerbound, max(left.upperbound, right.upperbound))
         elif left:
             return left
         else:
@@ -414,32 +401,19 @@ class TestRange(DomainCase):
         range_, member = args
         self.assertIn(member, range_)
 
-    @given(
-        float_ranges(kinds=kinds_with_samples)
-        | date_ranges(kinds=kinds_with_samples)
-    )
+    @given(float_ranges(kinds=kinds_with_samples) | date_ranges(kinds=kinds_with_samples))
     def test_sample_in_range(self, range_):
-        self.assertIn(
-            range_.sample, range_, f"Sample {range_.sample} is not in {range_}"
-        )
+        self.assertIn(range_.sample, range_, f"Sample {range_.sample} is not in {range_}")
 
     @given(float_ranges() | date_ranges())
     def test_range_kinds(self, range_):
-        if isinstance(range_._lower, Included) and isinstance(
-            range_._upper, Excluded
-        ):
+        if isinstance(range_._lower, Included) and isinstance(range_._upper, Excluded):
             self.assertTrue(range_.kind == RangeKind.OPEN_RIGHT)
-        elif isinstance(range_._lower, Included) and isinstance(
-            range_._upper, Included
-        ):
+        elif isinstance(range_._lower, Included) and isinstance(range_._upper, Included):
             self.assertTrue(range_.kind == RangeKind.CLOSED)
-        elif isinstance(range_._lower, Excluded) and isinstance(
-            range_._upper, Excluded
-        ):
+        elif isinstance(range_._lower, Excluded) and isinstance(range_._upper, Excluded):
             self.assertTrue(range_.kind == RangeKind.OPEN)
-        elif isinstance(range_._lower, Excluded) and isinstance(
-            range_._upper, Included
-        ):
+        elif isinstance(range_._lower, Excluded) and isinstance(range_._upper, Included):
             self.assertTrue(range_.kind == RangeKind.OPEN_LEFT)
 
     @given(st.floats(allow_nan=False) | st.datetimes() | st.integers())
@@ -582,10 +556,7 @@ class IntervalTest(DomainCase):
         interval = IntervalSet(ranges)
         self.assertSampleIsMember(interval, interval)
 
-    @given(
-        st.lists(float_ranges(), max_size=10)
-        | st.lists(date_ranges(), max_size=10)
-    )
+    @given(st.lists(float_ranges(), max_size=10) | st.lists(date_ranges(), max_size=10))
     def test_interval_filters_emptys(self, ranges):
         interval = IntervalSet(ranges)
         for range_ in interval.ranges:
@@ -605,9 +576,7 @@ class IntervalTest(DomainCase):
         self._test_interval_diff_produces_disjoint_ranges(r1, r2)
 
     @given(st.lists(float_ranges(), min_size=2, max_size=10), st.data())
-    def test_float_interval_diff_produces_disjoint_ranges_non_empty_diff(
-        self, r1, data
-    ):
+    def test_float_interval_diff_produces_disjoint_ranges_non_empty_diff(self, r1, data):
         last_range = r1[-1]
         next_range = _build_next_range(last_range, 1, data)
         r2 = r1 + [next_range]
@@ -624,9 +593,7 @@ class IntervalTest(DomainCase):
         i1 = IntervalSet(r1)
         i2 = IntervalSet(r2)
         idiff = i1.diff(i2)
-        assert idiff.ranges == tuple(
-            sorted(idiff.ranges, key=operator.attrgetter("lowerbound"))
-        )
+        assert idiff.ranges == tuple(sorted(idiff.ranges, key=operator.attrgetter("lowerbound")))
 
     @given(
         st.lists(float_ranges(), min_size=2, max_size=10),
@@ -643,9 +610,7 @@ class IntervalTest(DomainCase):
         self._test_interval_diff_produces_sorted_ranges(r1, r2)
 
     @given(st.lists(date_ranges(), min_size=2, max_size=10), st.data())
-    def test_date_interval_diff_produces_sorted_ranges_non_emptydiff(
-        self, r1, data
-    ):
+    def test_date_interval_diff_produces_sorted_ranges_non_emptydiff(self, r1, data):
         last_range = r1[-1]
         next_range = _build_next_range(last_range, DAY, data)
         r2 = r1 + [next_range]
@@ -655,10 +620,9 @@ class IntervalTest(DomainCase):
         i1 = IntervalSet(r1)
         i2 = IntervalSet(r2)
         idiff = i1.diff(i2)
-        assert not idiff.ranges or idiff.ranges == idiff.ranges[0].join(
-            *idiff.ranges[1:]
-        )
+        assert not idiff.ranges or idiff.ranges == idiff.ranges[0].join(*idiff.ranges[1:])
 
+    @settings(deadline=None)
     @given(
         st.lists(float_ranges(), min_size=2, max_size=10),
         st.lists(float_ranges(), max_size=10),
@@ -666,10 +630,9 @@ class IntervalTest(DomainCase):
     def test_float_interval_diff_produces_non_toucheing_ranges(self, r1, r2):
         self._test_interval_diff_produces_non_toucheing_ranges(r1, r2)
 
+    @settings(deadline=None)
     @given(st.lists(float_ranges(), min_size=2, max_size=10), st.data())
-    def test_float_interval_diff_produces_non_toucheing_ranges_non_emptydiff(
-        self, r1, data
-    ):
+    def test_float_interval_diff_produces_non_toucheing_ranges_non_emptydiff(self, r1, data):
         last_range = r1[-1]
         next_range = _build_next_range(last_range, 1, data)
         r2 = r1 + [next_range]
@@ -726,6 +689,12 @@ class IntervalTest(DomainCase):
         self.assertEqual(i1, i2)
 
 
+@given(get_date_member_strategy())
+def test_get_date_member_strategy_produces_nonimaginary_unfolded_dates(d):
+    assert getattr(d, "fold", 0) == 0
+    assert getattr(d, "tzinfo", None) is None
+
+
 def _build_next_range(r, delta, data):
     Bound1, Bound2 = data.draw(st.tuples(type_of_bounds, type_of_bounds))
     lowerbound = r.upperbound + delta
