@@ -24,9 +24,7 @@ from .strategies.domains import (
     members_of_float_ranges,
 )
 
-MORE_THAN_ONE_LIFETIME = Range.new_open_right(
-    datetime(1978, 10, 21), datetime(9078, 10, 21)
-)
+MORE_THAN_ONE_LIFETIME = Range.new_open_right(datetime(1978, 10, 21), datetime(9078, 10, 21))
 
 
 @given(st.data(), many_date_ranges(5, outer=MORE_THAN_ONE_LIFETIME))
@@ -34,9 +32,7 @@ def test_date_interval_trees_finds_the_right_interval(data, intervals):
     cells = [Cell.from_range(r, i) for i, r in enumerate(intervals)]
     tree = IntervalTree.from_cells(cells)
     picked_cell = data.draw(st.sampled_from(cells))
-    interval = Range.new_open_right(
-        picked_cell.lowerbound, picked_cell.upperbound
-    )
+    interval = Range.new_open_right(picked_cell.lowerbound, picked_cell.upperbound)
     member = data.draw(members_of_date_ranges(interval))
     assert tree[member] == (picked_cell,)
     for protocol in range(pickle.DEFAULT_PROTOCOL, pickle.HIGHEST_PROTOCOL):
@@ -66,9 +62,7 @@ def test_float_interval_trees_finds_the_right_interval(data, intervals):
     cells = [Cell.from_range(r, i) for i, r in enumerate(intervals)]
     tree = IntervalTree.from_cells(cells)
     picked_cell = data.draw(st.sampled_from(cells))
-    interval = Range.new_open_right(
-        picked_cell.lowerbound, picked_cell.upperbound
-    )
+    interval = Range.new_open_right(picked_cell.lowerbound, picked_cell.upperbound)
     member = data.draw(members_of_float_ranges(interval))
     assert tree[member] == (picked_cell,)
     for protocol in range(pickle.DEFAULT_PROTOCOL, pickle.HIGHEST_PROTOCOL):
@@ -77,9 +71,7 @@ def test_float_interval_trees_finds_the_right_interval(data, intervals):
 
 
 @given(st.data(), many_float_ranges(5, outer=FLOAT_OUTER_RANGE))
-def test_float_interval_trees_finds_the_right_interval_issue3_regression(
-    data, intervals
-):
+def test_float_interval_trees_finds_the_right_interval_issue3_regression(data, intervals):
     cells = [Cell.from_range(r, i) for i, r in enumerate(intervals)]
     # The following two cells are regression tests for issue
     # https://gitlab.merchise.org/mercurio-2018/xotless/-/issues/3
@@ -87,9 +79,7 @@ def test_float_interval_trees_finds_the_right_interval_issue3_regression(
     cells.append(Cell.from_bounds(FLOAT_UPPER_LIMIT + 1, None, len(cells)))
     tree = IntervalTree.from_cells(cells)
     picked_cell = data.draw(st.sampled_from(cells))
-    interval = Range.new_open_right(
-        picked_cell.lowerbound, picked_cell.upperbound
-    )
+    interval = Range.new_open_right(picked_cell.lowerbound, picked_cell.upperbound)
     member = data.draw(members_of_float_ranges(interval))
     assert tree[member] == (picked_cell,)
     for protocol in range(pickle.DEFAULT_PROTOCOL, pickle.HIGHEST_PROTOCOL):
@@ -112,9 +102,7 @@ def test_float_interval_trees_finds_only_the_right_intervals(data, intervals):
     cells = [Cell.from_range(r, i) for i, r in enumerate(intervals)]
     tree = IntervalTree.from_cells(cells)
     picked_cell = data.draw(st.sampled_from(cells))
-    interval = Range.new_open_right(
-        picked_cell.lowerbound, picked_cell.upperbound
-    )
+    interval = Range.new_open_right(picked_cell.lowerbound, picked_cell.upperbound)
     member = data.draw(members_of_float_ranges(interval))
     expected = {cell for cell in cells if member in cell}
     assert set(tree[member]) == expected
@@ -128,9 +116,7 @@ def test_float_interval_trees_finds_only_the_right_intervals(data, intervals):
 @given(
     st.data(),
     st.lists(
-        date_ranges(
-            outer=MORE_THAN_ONE_LIFETIME, kinds=st.just(Range.new_open_right)
-        ),
+        date_ranges(outer=MORE_THAN_ONE_LIFETIME, kinds=st.just(Range.new_open_right)),
         min_size=1,
         max_size=10,
     ),
@@ -139,9 +125,7 @@ def test_date_interval_trees_finds_only_the_right_intervals(data, intervals):
     cells = [Cell.from_range(r, i) for i, r in enumerate(intervals)]
     tree = IntervalTree.from_cells(cells)
     picked_cell = data.draw(st.sampled_from(cells))
-    interval = Range.new_open_right(
-        picked_cell.lowerbound, picked_cell.upperbound
-    )
+    interval = Range.new_open_right(picked_cell.lowerbound, picked_cell.upperbound)
     member = data.draw(members_of_date_ranges(interval))
     expected = {cell for cell in cells if member in cell}
     assert set(tree[member]) == expected
