@@ -74,9 +74,7 @@ def test_line_back_inverse_of_next(instances, pos):
         try:
             assert r.next() == previous
         except StopIteration:
-            assert (
-                False
-            ), "If I can go back in l, I must be able to forward in ~l"
+            assert False, "If I can go back in l, I must be able to forward in ~l"
     except StopIteration:
         pass
 
@@ -90,9 +88,7 @@ def test_line_next_inverse_of_back(instances, pos):
         try:
             assert r.back() == following
         except StopIteration:
-            assert (
-                False
-            ), "If I can go forward in l, I must be able to back in ~l"
+            assert False, "If I can go forward in l, I must be able to back in ~l"
     except StopIteration:
         pass
 
