@@ -73,7 +73,7 @@ class Cell(t.Generic[T, TOrd]):
 
 
 @dataclass(init=False, repr=False)
-class IntervalTree:
+class IntervalTree(t.Generic[T, TOrd]):
     """A data-containing generic IntervalTree.
 
     Intended usage:
@@ -105,13 +105,16 @@ class IntervalTree:
 
     """
 
-    _impl: _ORIntervalTree = field(init=False)
+    _impl: _ORIntervalTree[T, TOrd] = field(init=False)
 
     def __init__(self, cells: t.Sequence[Cell[T, TOrd]]) -> None:
         self._impl = _ORIntervalTree.from_cells(cells)
 
     @classmethod
-    def from_cells(cls, cells: t.Sequence[Cell[T, TOrd]]) -> IntervalTree:
+    def from_cells(
+        cls,
+        cells: t.Sequence[Cell[T, TOrd]],
+    ) -> IntervalTree[T, TOrd]:
         """Build an IntervalTree from the given sequence of cells."""
         return cls(cells)
 
@@ -154,19 +157,15 @@ class _ORIntervalTree(_IntervalTreeImplementation[T, TOrd]):
 
     center: t.Optional[TOrd]
     cells: t.Sequence[Cell[T, TOrd]]  # Ordered by lowerbound
-    cells_by_upper: t.Sequence[
-        Cell[T, TOrd]
-    ]  # Ordered in reverse by upperbound.
+    cells_by_upper: t.Sequence[Cell[T, TOrd]]  # Ordered in reverse by upperbound.
     left: t.Optional[_ORIntervalTree]
     right: t.Optional[_ORIntervalTree]
 
     __slots__ = ("center", "cells", "cells_by_upper", "left", "right")
 
     @classmethod
-    def from_cells(cls, cells: t.Sequence[Cell[T, TOrd]]) -> _ORIntervalTree:
-        return cls._from_sorted_cells(
-            sorted(tuple(cell for cell in cells if cell), key=_getlower)
-        )
+    def from_cells(cls, cells: t.Sequence[Cell[T, TOrd]]) -> _ORIntervalTree[T, TOrd]:
+        return cls._from_sorted_cells(sorted(tuple(cell for cell in cells if cell), key=_getlower))
 
     def __bool__(self) -> bool:
         return bool(self.cells)
@@ -200,13 +199,9 @@ class _ORIntervalTree(_IntervalTreeImplementation[T, TOrd]):
             result = cls(
                 center=center,
                 cells=tuple(middle_cells),
-                cells_by_upper=tuple(
-                    sorted(middle_cells, key=_getupper, reverse=True)
-                ),
+                cells_by_upper=tuple(sorted(middle_cells, key=_getupper, reverse=True)),
                 left=cls._from_sorted_cells(left_cells) if left_cells else None,
-                right=cls._from_sorted_cells(right_cells)
-                if right_cells
-                else None,
+                right=cls._from_sorted_cells(right_cells) if right_cells else None,
             )
             return result
 
@@ -234,9 +229,7 @@ class _ORIntervalTree(_IntervalTreeImplementation[T, TOrd]):
             yield from takewhile(lambda c: c.lowerbound <= query, self.cells)
         else:
             # The same as before but with `cells_by_upper`.
-            yield from takewhile(
-                lambda c: query < c.upperbound, self.cells_by_upper
-            )
+            yield from takewhile(lambda c: query < c.upperbound, self.cells_by_upper)
         if query < self.center and self.left:
             yield from self.left.get(query)
         elif query > self.center and self.right:
@@ -255,3 +248,19 @@ _getlower = operator.attrgetter("lowerbound")
 _getupper = operator.attrgetter("upperbound")
 
 sentry_span = get_module_sentry_spanner(__name__)
+
+
+if t.TYPE_CHECKING:
+    from datetime import datetime
+
+    Cell(1.0, 1.0, "Something")
+    IntervalTree.from_cells([Cell(1.0, 1.0, "float"), Cell(1.0, 2.0, "float")])
+    IntervalTree.from_cells([Cell(1, 1, "int"), Cell(-101, 21, "int")])
+    IntervalTree.from_cells(
+        [
+            Cell(datetime(2007, 3, 22), datetime(2022, 3, 22), "dt"),
+            Cell(datetime(1978, 10, 21), datetime(2022, 10, 21), "dt"),
+        ]
+    )
+
+    IntervalTree.from_cells([Cell(1.0, 1.0, "float"), Cell(1, 2, "int")])  # type: ignore
